/* Programme de base pouvant servir à modifier BOLOMETRE.ino.

Cette version du programme de base intègre :
• La page graphique avec le PAPILLON,
• Le mode SOMMEIL avec gestion ÉTÉ et HIVER,
• Le P.E.P. avec possibilité du RESET froid et du RESET chaud. (Inverseur sur D8.)

B.P. court sur rouge : Passe à l'ITEM suivant dans le MENU.
B.P. court sur noir : Passe à l'ITEM précédent dans le MENU.
B.P. rouge long : Fonction de recalibrage rapide de l'Heure.
B.P. noir long : Allume ou éteint l'écran OLED.

*----- Programme de base avec le PAPILLON, le P.E.P. et le mode SOMMEIL -----
 Programme : 27228 octets 88%  Mémoire dynamique :  1192 octets 58%
*/

//--------------------- Déclarations pour le matériel -----------------------
#include "U8glib.h"
U8GLIB_SH1106_128X64 u8g(U8G_I2C_OPT_NONE); // Usage de la ligne I2C (Two Wire Interface)

#include <avr/sleep.h>
#include <Wire.h>
#include <Rtc_Pcf8563.h>
// Initialisation de l'horloge temps réel.
Rtc_Pcf8563 rtc;
//--------------------- Définition des broches d'E/S ------------------------
#define BUZZER 2 // Pilotage du bruiteur.
// NOTE : pas besoin de déclarer D3 car traitée par la bibliothèque de l'Horloge/Calendrier.
#define Sommeil_interdit 4 // Entrée binaire D4 pour Sommeil autorisé.
#define Pilotage_T1 5
#define Pilotage_T2 6
#define Presence_Strap 7 // Entrée binaire D7 pour tester la présence d'un "Strap".
#define RESET_froid 8 // Entrée binaire D8 pour choisir entre RESET Froid ou P.E.P.
#define INV_piles 9  // Inverseur de mesures de la tension sur les piles..
#define LED_Clavier 10 // Témoin d'activation d'un B.P.
#define LED_Rouge 11  // LED B.P. long.
#define LED_bleue 12 // Témoin de la boucle de base.

#define Entree_U_Solaire 1 // Entrée analogique A1 pour mesurer la tension cellules solaires.
#define Entree_U_apres_10ohm 2 // Entrée analogique A2 pour mesurer la tension après la 10ohm.
#define Entree_U_Pile_Horloge 3 // Entrée analogique A3 pour mesurer la tension pile de sauvegarde.
#define Entree_U_Accumulateur 6    // Entrée analogique A6 pour mesurer la tension sur l'accumulateur.
#define Entree_U_NANO 7    // Entrée analogique A7 pour mesurer la tension sur la carte NANO.

//-------------------- Déclaration des constantes --------------------
#define Version_PGM "Version 16-03-19" //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#define Correction_U_Solaire 0.02105 // Corrige la valeur du pont diviseur pour U cellules solaires.
// Voir explications pour cette valeur dans la fiche logicielle spécifique.
#define Correction_U_apres_R 0.02072 // Corrige la valeur du pont diviseur pour U mesure intensité totale.
// Voir explications pour cette valeur dans la fiche logicielle spécifique.
#define Correction_U_Pile_sauvegarde 0.0046887 // Pile de sauvegarde de l'horloge. (0.0044965 = 0.92 x 5 / 1023)
#define Correction_U_Arduino 0.0098087 // Corrige la valeur du pont diviseur pour U carte NANO.
#define Correction_U_Accumulateur 0.009775 // Corrige la valeur du pont diviseur pour U Accumulateur.
#define Correction_Shunt 86.1111 // Corrige la valeur du shunt de 10ohm qui mesure I Totale.
// Ci dessus la valeur est * par 100 car divisé par 10ohm puis 1000 pour des mA
#define Resistance_de_charge 0.136 // Valeur exacte de deux résistances de 68ohm en série pour la charge.
#define T_base_millisecondes 998; // // Valeur affinée en fonction de la QRG du quartz de la carte Arduino.
#define NB_tests_antirebonds 50 // Nombre de tests de stabilisation B.P.
//-------------------- Déclaration des variables ---------------------
boolean Clic_sur_le_BP, Augmenter_le_choix, Clic_long;
boolean HIVER; // Gère la commutation de T1 et de T2.
boolean Mode_VEILLE;
boolean Rafraichir_ecran;

byte Compteur_HIVER;
byte Decalage;
byte I_Carte_NANO; // Intensité consommée par la carte NANO Arduino.
byte Item;
byte Num_de_la_semaine;
byte Octet_EEPROM;

byte Heures, Minutes, Secondes, Jour, Mois, J_Semaire, Annee;

volatile byte Drapeau = 1;
ISR(WDT_vect){if (Drapeau == 0) Drapeau = 1;} // Routine d'interruption pour le mode sommeil.
  
char JMMJVSD[8] = {'L','M','M','J','V','S','D'};

int PTR; // Pointeur pour échanger des Octets en EEPROM.

unsigned long Ancienne_valeur_Temps_Ecoule_depuis_RESET;

float I_Total_solaire; // courant totalfourni par les panneaux solaires.
float I_Recharge; // Intensité de recharge d'un accumulateurs.
float I_sur_charge; // Intensité dans les résistances de charge.
float U_Accumulateur; // Tension U aux bornes d'un accumulateur.
float U_Arduino; // Tension U aux bornes de la carte Arduino.
float U_Pile_Horloge; // Tension U de la pile de sauvegarde horloge.
float U_Solaire; // Tension U aux bornes des résistances de charge.
float U_Solaire_apres_shunt;
float Valeur_mesuree;
float W_Echantillon;
float W_Journaliere;

// L'image est une matrice de 67 lignes de 8 octets soit 4288 Pixels.
// Soit une taille de 536 octets.

#define Largeur_du_dessin 64  // Impose la largeur de la matrice en pixels.
#define Hauteur_du_dessin 67 // Impose la Hauteur de la matrice en pixels.
  
byte DessinBitMap[] = {
     B11110000,1,B11110000,0,0,B00111000,B11101110,0, B11011000,2,B11011000,1,0,B01001000,B01000001,0,
     B11110100,12,B11111100,2,0,B00111000,B01000110,0, B11111100,B00011111,B11111100,7,0,B00101000,B01001000,0,
     254,B00101111,B11111010,B00000101,0,B01001000,B01000111,0, B00111110,B01101111,B11111010,15,0,0,0,0,
     B10101011,B11011011,127,14,0,0,0,0,B11110111,B11110011,B01111110,B00011010,0,0,0,0,
     B11110011,B01111011,B01111111,B00011010,0,0,0,0,B11111101,B11111111,B11000111,
     B00111111,0,0,0,0,B11111111,B11011111,B11100111,B01110111,0,0,0,0,B11111101,B10011111,
     B11111100,B11011101,0,0,0,0,255,255,255,B11111000,1,0,0,0,255,255,255,B11101100,3,0,0,
     0,255,B01111111,B11001111,B11000101,7,0,0,0,255,254,B01111111,B11000110,15,0,0,0,255,
     B11000111,B01111101,B11011010,B00010110,0,0,0,255,B11000111,253,B01100001,B00011100,0,
     0,0,255,B11101111,255,B10110011,B00011100,0,0,0,B00101111,254,B11110111,B00011101,
     B00011111,0,0,0,B11001110,254,B11110111,B00011111,14,0,0,0,14,255,B00111111,B00010111,
     15,0,0,0,254,255,255,255,15,0,0,0,B11111100,255,254,255,7,0,0,0,B11100100,B01111111,254,B10111011,
     7,B00011111,0,0,B11100100,B1011111,B10011100,255,B11000011,B11101110,0,0,B11111100,B00111011,B00001101,
     254,B11100011,255,1,0, B11111000,B11010011,255,254,
     B00011001,254,7,0,B01111000,B11010011,255,B11010110,15,B11100011,15,0,B01110000,B11010010,255,B0001011,
     B10011111,B11000001,B00011111,0,B01110000,254,255,B00100011,B11011111,B11100000,B00111101,0,B10010000,
     B10111110,255,B01010011,B01111111,B00110000,B00111000,0,B10100000,B01111111,B11111011,B01111101,255,
     B00011111,B01111000,0,B00100000,255,255,255,B00011111,B01111100,B11110000,0,B11000000,255,253,255,15,
     B10000011,B01111110,1,B11000000,B11011111,253,255,B10001111,B11000001,255,1,B10000000,B10101111,253,255,
     B11100111,B11110000,B11100010,3,B10000000,B10101111,B11111001,253,255,B00111111,B1110100,7,0,B11001111,
     B11110011,252,7,B00011100,B11001000,7,0,B11001111,B00110011,B11110001,7,254,B11010000,B00001011,0,B11011110,
     B00111011,B11110001,B11011011,B01111001,B11010111,15,0,B11011100,B10011111,B11111010,B11100001,B01111111,
     B11011111,B00001011,0,252,B11011111,253,B11100001,255,B11101111,B00011111,0,252,B11001111,255,B00111111,254,
     255,B00011111,B01111100,B11111000,B11011111,255,B11011110,B10011100,255,B00011111,B10000011,B01111001,
     B11101100,255,B00001100,B01101111,255,B00111111,0,B01110010,B11001100,255,4,B1000111,254,B00111111,
     0,B11110010,B11011111,255,B10000111,B10011001,B11000111,B00111111,B01100000,B11100100,253,255,B11111001,
     B11100000,B10000011,B00111011,B10011000,B11010101,B11011101,255,B01110001,B01110000,B10000100,B00111111,
     6,B11111010,B01011011,B10111110,B10011000,B10011101,B10001000,B00111111,1,252,B00111111,B10011110,4,7,255,
     B01111111,0,252,B00111111,B10011111,B11101111,B11001111,B00010011,B01111111,0,252,B10111111,B11001111,
     B00010000,B11111000,B00100010,B01111110,0,252,255,B01000111,B01111100,B01100000,B00100010,B10011110,
     0,B11111000,255,B11000011,255,B11111000,B01000111,B10111110,0,B10000000,255,B00110011,B11111000,
     B01011110,B01111110,B10111110,0,0,254,B11001111,B11000000,B10000011,B11001000,255,0,0,252,7,255,255,
     B10001000,254,0,B11000000,255,B00011111,B00011100,B11100000,B10001111,252,0,B00110000,B11100000,B00011111,
     B01100000,0,B10111101,254,0,8,B11000000,B01111111,B10000000,3,B11010001,255,0,4,B11111100,255,0,B00111100,
     B00010001,255,0,2,2,254,3,B01100000,B00010011,B01111101,0,1,1,255,15,B11000000,B10110011,B00101101,
     B10000000,B10000000,B11100000,B11110000,B00011111,0,B11011100,B00011111,
     0,B01000000,B00010000,B11100000,B00001111,0,B11100000,7,};

void setup() {
  //+++++++++++++++ Initialiser_le_chien_de_garde pour le mode sommeil ++++++++++++++++
  MCUSR &= ~(1 << WDRF); // Annuler de drapeau de RESET.
  WDTCSR |= (1 << WDCE) | (1 << WDE); // Séquence de démarrage.
  WDTCSR = 49; // Initialiser la nouvelle valeur pour le "Timeout" pour 8,444s.
  WDTCSR |= _BV(WDIE);
  // ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  Initialiser_les_valeurs_temporelles();
  Octet_EEPROM = Lire_un_Octet_en_EEPROM(50); Octet_EEPROM++; Ecrire_un_Octet_en_EEPROM(50, Octet_EEPROM);
  Num_de_la_semaine = Calcule_le_Num_de_la_semaine(); W_Echantillon = 0;
  u8g.setFont(u8g_font_profont12); // Police de matrices de 6x10 pixels lettres MAJ/min comprises.
  u8g.sleepOn();
  W_Journaliere = 0;  
  // ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  pinMode(LED_bleue, OUTPUT); pinMode(BUZZER, OUTPUT); digitalWrite(BUZZER, LOW); pinMode(LED_Rouge, OUTPUT);
  digitalWrite(LED_Rouge, LOW); pinMode(LED_Clavier, OUTPUT);  digitalWrite(LED_Clavier, LOW);
  pinMode(Pilotage_T1, OUTPUT);  digitalWrite(Pilotage_T1, LOW); pinMode(Pilotage_T2, OUTPUT);
  digitalWrite(Pilotage_T2, LOW); pinMode(Presence_Strap, INPUT);
  digitalWrite(Presence_Strap, HIGH); // Valide la résistance de pullup.
  pinMode(RESET_froid, INPUT); digitalWrite(RESET_froid, HIGH); // Valide la résistance de pullup.
  pinMode(Sommeil_interdit, INPUT); digitalWrite(Sommeil_interdit, HIGH); // Valide la résistance de pullup.
  pinMode(INV_piles, INPUT); digitalWrite(INV_piles, HIGH); // Valide la résistance de pullup.
  // ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  Mode_VEILLE = true; // Forcer l'écran noir si RESET de type P.E.P.
  if (digitalRead(RESET_froid)) {// Si inverseur de RESET ouvert effectuer un démarrage complet.
     PAGE0();  BIP(); ATTENDRE_un_BP(); //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
     if (Clic_long && Augmenter_le_choix) Remise_a_ZERO_des_donnees();
     Mode_VEILLE = false; // Pour qu'en sortie les écrans soient affichés.
     // Si le strap n'est pas sur D7 procéder à l'initialisation du Num de la semaine.
     if (digitalRead(Presence_Strap)) Initialiser_le_Num_de_la_semaine();}        
  // ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  //@@@@@@@@@@@@@ Ci-dessous code ajouté pour afficher la place disponible. @@@@@@@@@@@
  // u8g.firstPage(); do {u8g.setPrintPos(22, 35); u8g.print("PILE - TAS = ");
  // u8g.print(SRAM_LIBRE());} while(u8g.nextPage()); ATTENDRE_un_BP();
  //@@@@@@@@@@@@@ ******************************************************** @@@@@@@@@@@@
  Item = 1;}
  
//@@@@@@@@@@@@@@ Ci-dessous fonction pour afficher la place disponible @@@@@@@@@@@@@@
//  int SRAM_LIBRE() { // Fonction qui retourne la taille de SRAM disponible.
//  extern int __heap_start, *__brkval; // Déclaration des deux pointeurs dédiés.
//  byte BIDON; // Dernière variable allouée, donc occupe le "haut" du TAS.
//  if (__brkval == 0) {return (int) &BIDON -(int) &__heap_start;}
//  else {return (int) &BIDON -(int) __brkval;} }
//@@@@@@@@@@@@@@ ***************************************************** @@@@@@@@@@@@@@

void loop() {
  HIVER = (rtc.getMonth() < 5) ||  (rtc.getMonth() > 9); // Placé ici car utilisé deux fois dans loop.
  // +++++++++++++++++ Si horaires nocturnes valider le mode sommeil. +++++++++++++++++
  Gerer_le_mode_sommeil();
  // ++++++++++++++++++++++++++ Réalisé une fois par seconde. +++++++++++++++++++++++++
  if (millis() > Ancienne_valeur_Temps_Ecoule_depuis_RESET)  {
      Ancienne_valeur_Temps_Ecoule_depuis_RESET = millis() + T_base_millisecondes; // Réarme le processus.
      // --------------- Traiter la saturation du compteur  millis(). -----------------
      if ((Ancienne_valeur_Temps_Ecoule_depuis_RESET - millis()) > 10000)
         Ancienne_valeur_Temps_Ecoule_depuis_RESET = millis() + T_base_millisecondes; // Réarme le processus.
      //-------------------------------------------------------------------------------
      digitalWrite(LED_bleue, HIGH); delay(3); digitalWrite(LED_bleue, LOW);
      // ------------------ Traiter la commutation de T1 et de T2. --------------------
      Traiter_la_commutation_de_T1_et_de_T2();
      // -------------------- Si 00H 00S passage au jour suivant. --------------------- 
      Passer_au_Jour_Suivant();
      // --------- Si 10 minutes écoulées enregistrer un échantillon dans Jr. ---------
      Enregistrer_toutes_les_10_minutes();
      //-------------------------------------------------------------------------------
      if ((!Mode_VEILLE) && (Rafraichir_ecran)) Afficher_sur_OLED();}
  // +++++++++++++++++++++ Traiter la requête de l'un des deux B.P. +++++++++++++++++++
  Tester_le_BP_A0();
  if (Clic_sur_le_BP) {Clic_sur_le_BP = false; Traiter_le_clavier();}} // Action sur le B.P. prise en compte.
  //============================== Fin de la boucle de base ===========================
  
void Initialiser_les_valeurs_temporelles() {
  Heures = rtc.getHour(); Minutes = rtc.getMinute(); Secondes = rtc.getSecond();
  Jour = rtc.getDay(); Mois = rtc.getMonth(); Annee = rtc.getYear(); J_Semaire = rtc.getWeekday();}
  
void Gerer_le_mode_sommeil() { 
  // Entre 22H le soir et 7H le matin passer en Sommeil par séquences de cinq minutes l'ÉTÉ,
  // Entre 19H le soir et 8H le matin passer en Sommeil par séquences de cinq minutes l'HIVER,
  // Entre 0H et 1H ne pas passer en sommeil pour changer normalement de jour.
  byte Debut_Jour, Debut_Nuit;
  if (HIVER) Debut_Jour = 8; else Debut_Jour = 7; if (HIVER) Debut_Nuit = 18; else Debut_Nuit = 21;
  if (!digitalRead(Sommeil_interdit)) // Ne passer en Sommeil si le strap est enlevé.
     if (((rtc.getHour() > Debut_Nuit) || ((rtc.getHour() > 0) && (rtc.getHour() < Debut_Jour))))
         Mode_Sommeil();}

void Mode_Sommeil() {
  // Cette procédure engage le processeur en sommeil durant 5 minutes.
  // Chaque séquence dure 8,444 secondes environ.
  byte Nb_sequences_de_sommeil = 35; //35; // Chaque unité ajoutée augmentera de 8,444s la durée de veille.
  // Avec la valeur 35 on aura un sommeil de 8.444 x 35 = 295s soit environ 5 minutes.
  u8g.firstPage(); do {} while(u8g.nextPage()); // Éteindre l'écran si allumé sur RESET "froid".
  delay(3); set_sleep_mode(SLEEP_MODE_PWR_DOWN); // Définition du mode de veille désiré.
  // NOTE : Parmi les cinq modes de veille, PWR_DOWN est le plus économe.
  while (Nb_sequences_de_sommeil > 0) {
    sleep_enable(); // Arme le drapeau qui autorise la veille.
    sleep_cpu(); // Active effectivement la veille.
    // Entre dans le mode veille choisi pour 8 secondes.
    // Le processeur se réveille ici après le "Timeout du WDT".
    sleep_disable(); // Première action : Désactiver le mode sommeil.
    Nb_sequences_de_sommeil--;}}
    
void Traiter_le_clavier() {
  Rafraichir_ecran = true;
  if (Clic_long) {if (Augmenter_le_choix) {
                      if (digitalRead(Presence_Strap)) Synchroniser_les_secondes();
                      else  {Recopie_90_Octets(154,334); Ecrire_un_Octet_en_EEPROM(52,Jour);
                             Ecrire_un_Octet_en_EEPROM(53,Mois);}}
                  else {Mode_VEILLE = !Mode_VEILLE; if (Mode_VEILLE) u8g.sleepOn(); else u8g.sleepOff();}}
  else if (Augmenter_le_choix) {Item++; if (Item == 13) Item = 1;}
       else if (Item > 1) Item--; else Item = 12;}
  
void Traite_le_debordement_de_365J() {
  ATTENDRE_un_BP(); PAGE_Debordement();  BIP(); ATTENDRE_un_BP();
  if ((Clic_long) && (Augmenter_le_choix)) Remise_a_ZERO_des_donnees();}
  
void PAGE_Debordement() {
  Reveiller_Ecran(); // Pour afficher si le mode veille est actif ce qui est en principe le cas.
  u8g.firstPage(); do {// Début de la boucle d'affichage de la page écran.
  //--------------------------------------------------------------
  Double_cadre(); Aff_TEXTE_X_Y(25, 23, 780, 15); Aff_TEXTE_X_Y(20, 45, 802, 16);}
  //--------------------------------------------------------------
  while(u8g.nextPage());} // Fin de la boucle d'affichage de la page écran.
  
void Reveiller_Ecran() {u8g.sleepOff(); Mode_VEILLE = false;} // Pour afficher si le mode veille est actif.

void Mesurer_la_DDP_sur_R_et_I_TOTAL() {
  //------------------- Mesure U avant le shunt 10ohm ------------------
  Mesurer_une_tension(Entree_U_Solaire); U_Solaire = Valeur_mesuree * Correction_U_Solaire;
  //------------------- Mesure U après le shunt 10ohm ------------------
  Mesurer_une_tension(Entree_U_apres_10ohm); U_Solaire_apres_shunt = Valeur_mesuree * Correction_U_apres_R;
  //----------- Calcule I TOTAL fourni par le panneau solaire ----------
  I_Total_solaire = (U_Solaire - U_Solaire_apres_shunt) * Correction_Shunt;
  if(I_Total_solaire < 120) I_Total_solaire = I_Total_solaire - 20; // Corrige les valeurs faibles.
  if (I_Total_solaire < 0) I_Total_solaire = 0;}
  
void Traiter_la_commutation_de_T1_et_de_T2() {
  Mesurer_la_DDP_sur_R_et_I_TOTAL(); digitalWrite(Pilotage_T1, LOW); // @ (Voir plus avant.
  if (U_Solaire < 12) {Compteur_HIVER = 0; digitalWrite(Pilotage_T2, LOW);}
  // Ci-dessus : Pilotage_T2 passé à zéro pour parer une chute brutale de Usolaire qui
  //             passerait en dessous de 12v rapidement alors qu'il y avait > 15v.
  if ((U_Solaire > 13) && (Compteur_HIVER < 20)) Compteur_HIVER++;
  if (Compteur_HIVER == 20) {
      digitalWrite(Pilotage_T1, HIGH); Mesurer_la_DDP_sur_R_et_I_TOTAL();
      digitalWrite(Pilotage_T2, LOW); // Placé ici au lieu de @ pour supprimer le clignottement.
      if (U_Solaire < 13) digitalWrite(Pilotage_T1, LOW);
      else {if (U_Solaire > 15) {
                digitalWrite(Pilotage_T2, HIGH); Mesurer_la_DDP_sur_R_et_I_TOTAL();
                if (U_Solaire < 15) digitalWrite(Pilotage_T2, LOW);}}}
  Mesurer_la_DDP_sur_R_et_I_TOTAL(); // Pour totaliser une valeur vraie.
  W_Echantillon = W_Echantillon + I_Total_solaire / 3600; // Augmente W échantillon au cours du temps en mAH.
  W_Journaliere = W_Journaliere + I_Total_solaire / 3600;} // Augmente le cumul au cours du temps en mAH.
  
void Passage_au_jour_suivant() {
  float Valeur;
    W_Echantillon = 0; 
  //------- Décale "à gauche" les 6 derniers jour de la semaine. -------
  PTR = 12;  // Décale "à gauche" les 6 derniers jour de la semaine;
  for(byte Nb = 0; Nb < 7; Nb++)
     {Valeur = Lire_un_Float_en_EEPROM(PTR); Ecrire_un_Float_en_EEPROM(PTR-4,Valeur);}
  //------- Remplace la valeur de W_journalière dans la semaine. -------  
  Ecrire_un_Float_en_EEPROM(32,W_Journaliere); // Jour -1 (Veille.)
  //-------------- Recopie 90 échantillons de Jr vers J-1. ------------
  Recopie_90_Octets(64,154);
  //--------- Mise à jour des valeurs journalières Mini / MAXI. --------
  if (W_Journaliere < Lire_un_Float_en_EEPROM(36)) // Traite la journée "minimale".
     if (W_Journaliere > 49) // Ne sauvegarde que si la valeur dépasse 50mAH pour la journée.
        {Ecrire_un_Float_en_EEPROM(36,W_Journaliere); // Ne sauvegarde que la valeur et la date.
         Ecrire_un_Octet_en_EEPROM(48,rtc.getDay()); Ecrire_un_Octet_en_EEPROM(49,rtc.getMonth());}
  if (W_Journaliere > Lire_un_Float_en_EEPROM(40)) // Traite la journée "MAXimale".
     {Ecrire_un_Float_en_EEPROM(40,W_Journaliere); Recopie_90_Octets(64,244);
      Ecrire_un_Octet_en_EEPROM(46,rtc.getDay()); Ecrire_un_Octet_en_EEPROM(47,rtc.getMonth());}
  //-------------- Ajouter "Veille J-1" à W Totale année. --------------
  Valeur_mesuree = Lire_un_Float_en_EEPROM(4) + W_Journaliere;
  Ecrire_un_Float_en_EEPROM(4, Valeur_mesuree);
  //---------- Mise à zéro des valeurs journalières courantes. ---------
  PTR = 64; for (byte I=0; I<90; I++) Ecrire_un_Octet_en_EEPROM(PTR,0); W_Journaliere = 0;
  //------------------ Traiter un changement de semaine ----------------
  if (rtc.getWeekday() == 1) {// Si le nouveau jour est un Lundi, traiter la semaine écoulée.
     Num_de_la_semaine = Calcule_le_Num_de_la_semaine(); Valeur = 0;
     for(PTR = 8; PTR < 36; PTR = PTR+4) Valeur = Valeur + Lire_un_Float_en_EEPROM(PTR);
     Ecrire_un_Float_en_EEPROM((424 + (4 * Num_de_la_semaine)), Valeur/1000);}
  //------- Incrémenter en EEPROM le nombre de jours enregistrés. ------
  PTR = Lire_un_Octet_en_EEPROM(44); PTR++; Ecrire_un_Entier_en_EEPROM(44,PTR);}
  
void Passer_au_Jour_Suivant() {
  // Ci-dessous 10 minutes, car durant les cinq premières le processeur est en sommeil.
  if ((rtc.getHour()==0) && rtc.getMinute()==10 && (rtc.getSecond()==0)) {
         Octet_EEPROM = Lire_un_Entier_en_EEPROM(44); Octet_EEPROM++;
         if (Octet_EEPROM < 366) {Ecrire_un_Entier_en_EEPROM(44,Octet_EEPROM); Passage_au_jour_suivant();}
         else Traite_le_debordement_de_365J();}}
         
void Enregistrer_toutes_les_10_minutes() {
  Initialiser_les_valeurs_temporelles();
  // L'enregistrement ne commencera qu'à partir de 7H ou 8H du matin jusqu'à 22H non comprise.
  if ((Heures > 6) && (Heures < 22)) {
      if (((Minutes==0) || (Minutes==10) || (Minutes==20) || (Minutes==30) || (Minutes==40) || (Minutes==50)) 
         && (Secondes==0)) Ecriture_Echantillon_en_EEPROM();}}
         
void Ecriture_Echantillon_en_EEPROM() {
  Decalage = byte(Minutes/10);
  PTR = int(((Heures - 7) * 6) + Decalage + 64); // Calculer le pointeur dans l'EEPROM.
  Ecrire_un_Octet_en_EEPROM(PTR, byte(W_Echantillon * 5)); W_Echantillon = 0;}        
         
void Initialiser_le_Num_de_la_semaine() {
  byte Jr_a_definir;
  Reveiller_Ecran(); Jr_a_definir = Lire_un_Octet_en_EEPROM(51);
  do { u8g.firstPage(); do {// Début de la boucle d'affichage de la page écran.
     //--------------------------------------------------------------
     Double_cadre(); Aff_TEXTE_X_Y(18, 20, 860, 5); u8g.print("du"); Aff_TEXTE(690, 6); Aff_TEXTE(784, 3);
     Aff_TEXTE_X_Y(24, 35, 681, 4); Aff_TEXTE(829, 7); AFF_N(); Affiche_caractere_degre(); AFF_2();
     Aff_TEXTE_X_Y(49, 50, 929, 5); u8g.print(Jr_a_definir);}
     //--------------------------------------------------------------
     while(u8g.nextPage()); // Fin de la boucle d'affichage de la page écran.
     ATTENDRE_un_BP(); if (Augmenter_le_choix) {Jr_a_definir++; if (Jr_a_definir > 8) Jr_a_definir = 1;}} 
  while (!((Clic_long) && (!Augmenter_le_choix)));
  Ecrire_un_Octet_en_EEPROM(51, Jr_a_definir); BIP();}
  
void Afficher_sur_OLED() {
    Mesure_les_parametres();   // Rafraichir_ecran = true : Affiche date et heure en haut de l'écran.
    Rafraichir_ecran = false;
    switch (Item) {
      case 1 : {Rafraichir_ecran = true; if (!Mode_VEILLE) PAGE1(); break;}
      case 2 : {Rafraichir_ecran = true; if (!Mode_VEILLE) PAGE2(); break;}
      case 3 : {Rafraichir_ecran = true; if (!Mode_VEILLE) PAGE3(); break;}
      case 4 : {Rafraichir_ecran = true; if (!Mode_VEILLE) PAGE4(); break;}
      case 5 : {PAGE5(); break;}
      case 6 : {PAGE6(); break;}
      case 7 : {PAGE7(); break;}
      case 8 : {PAGE8(); break;}
      case 9 : {PAGE9(); break;}
      case 10 : {PAGE10(); break;}
      case 11 : {PAGE11(); break;}
      case 12 :{PAGE12();}}}
      
//--------------------- Routines pour la gestion EEPROM ---------------------
void Aff_TEXTE_X_Y(byte X, byte Y, int PTR, byte Longueur) {
  u8g.setPrintPos(X, Y); Aff_TEXTE(PTR, Longueur);}

void Aff_TEXTE(int PTR, byte Longueur) {
  for (byte I=0; I<Longueur; I++) {u8g.print(char(Lire_un_Octet_en_EEPROM(PTR))); PTR++;}; }
  
byte Lire_un_Octet_en_EEPROM(int PTR) {
  return eeprom_read_byte((unsigned char*) PTR);}

void Ecrire_un_Octet_en_EEPROM(int ADRESSE, byte OCTET) { // Pointe l'octet libre suivant.
   eeprom_write_byte((unsigned char *) ADRESSE, OCTET); PTR++;}

int Lire_un_Entier_en_EEPROM(int ADRESSE) {return eeprom_read_word((unsigned int *) ADRESSE);}

void Ecrire_un_Entier_en_EEPROM(int ADRESSE, int MOT16Bits) { // Pointe l'octet libre suivant.
   eeprom_write_word((unsigned int *) ADRESSE, MOT16Bits); PTR++; PTR++;}

float Lire_un_Float_en_EEPROM(int ADRESSE) {return eeprom_read_float((float *) ADRESSE);}

void Ecrire_un_Float_en_EEPROM(int ADRESSE, float REEL) { // Pointe l'octet libre suivant.
   eeprom_write_float((float *) ADRESSE, REEL); PTR++; PTR++; PTR++; PTR++;}
//---------------------------------------------------------------------------
       
void Synchroniser_les_secondes() {
  u8g.sleepOff(); // Éventuellement forcer l'affichage si OLED en mode sommeil.
  u8g.firstPage(); do {// Début de la boucle d'affichage de la page écran.
  //------- Affiche la page écran d'initialisation rapide --------
  Encadrement_double(); Aff_TEXTE_X_Y(6, 26, 736, 4); u8g.print(" = 30");
  u8g.undoScale(); Aff_TEXTE_X_Y(11, 20, 802, 11); Aff_TEXTE(747, 7);
  Aff_TEXTE_X_Y(35, 32, 802, 4); Aff_TEXTE(740, 7);}
  //--------------------------------------------------------------
  while(u8g.nextPage()); // Fin de la boucle d'affichage de la page écran.
  ATTENDRE_un_BP();
  if (Augmenter_le_choix) {Heures = rtc.getHour(); Minutes = rtc.getMinute();
      if (Clic_long) Minutes++;
      rtc.setTime(Heures, Minutes, 30); // Dans l'ordre on impose : Heures, Minutes, Secondes.
      BIP();}  
  Item = 1; Mode_VEILLE = false;}

void BIP() {tone(BUZZER,3000,100); digitalWrite(BUZZER, LOW);}
  
void ATTENDRE_un_BP() { Clic_sur_le_BP = false;
  while (!Clic_sur_le_BP) {
    digitalWrite(LED_Clavier,!digitalRead(LED_Clavier)); // Inverser la LED clavier.
    delay(50); Tester_le_BP_A0();}
  digitalWrite(LED_Clavier,LOW); Clic_sur_le_BP = false; }
  
// >>>>>>>>>>>>>>> Le B.P. est câblé en "Analogique" <<<<<<<<<<<<<<<<<<
void Tester_le_BP_A0() { // La lecture sur A0 donne 0 ou 930 maximum.
  int Numerisation_BP; // Valeur retournée par la CAN de A0.
  byte COMPTEUR; unsigned long Horloge_actuelle;
  Numerisation_BP = analogRead(0);
  if ((Numerisation_BP > 710) || (Numerisation_BP < 240)) {
    Clic_sur_le_BP = true; digitalWrite(LED_Clavier, HIGH);
    Augmenter_le_choix = false;
    Horloge_actuelle = millis(); Clic_long = false;
    if (Numerisation_BP > 710) { Augmenter_le_choix = true; }
    // Attendre_la_stabilisation du travail durant 100mS. Bruiteur à 500Hz et laisse D2 à zéro :
    for(byte I=0; I < 50; I++) {digitalWrite(BUZZER, !digitalRead(BUZZER)); delay(1);}  
    // Attendre_la_stabilisation du repos :
    COMPTEUR = 0;
    while (COMPTEUR < NB_tests_antirebonds) {
      if ((millis() - Horloge_actuelle) > 700) {Clic_long = true; digitalWrite(LED_Rouge, HIGH);}
      COMPTEUR++; Numerisation_BP = analogRead(0);
      if ((Numerisation_BP > 710) || (Numerisation_BP < 240)) COMPTEUR = 0; }
  digitalWrite(LED_Clavier, LOW); digitalWrite(LED_Rouge, LOW);}}
  
void Trace_resistance(byte X, byte Y) {// Cout de ce tracé : 938 octets !
  u8g.drawRFrame(X,Y-7,43,8,3);X=X+4;
  for (byte I=0; I<3; I++) {u8g.drawLine(X,Y,X+6,Y-7); u8g.drawLine(X+6,Y-7,X+12,Y); X=X+12;}}
  
void Mesure_U_Pile_Horloge() {
  U_Pile_Horloge = analogRead(Entree_U_Pile_Horloge); // L'entier de numérisation passe dans le réel U_Solaire
  U_Pile_Horloge = (U_Pile_Horloge * Correction_U_Pile_sauvegarde);} // 0.0044965 = 0.92 x 5 / 1023.
 
void Mesurer_une_tension(byte Entre_utilisee) {  // Intègre 50 mesures antiparasites.
  Valeur_mesuree = 0;
  for (int I = 0; I < 50 ; I++) // Cumuler 50 mesures. 
      {Valeur_mesuree = Valeur_mesuree + (analogRead(Entre_utilisee)); }
  Valeur_mesuree = Valeur_mesuree / 50;}
  
void Mesurer_la_tension_cellules_photovoltaiques() {
  Mesurer_une_tension(Entree_U_apres_10ohm); U_Solaire = Valeur_mesuree * Correction_U_Solaire;}

void Mesure_les_parametres() {
  //------------------------- Mesure U Solaire -------------------------
  Mesurer_la_DDP_sur_R_et_I_TOTAL(); // Placé en procédure car utile à la boucle de base.
  //----------------------- Mesure U Accumulateur ----------------------  
  Mesurer_une_tension(Entree_U_Accumulateur); U_Accumulateur = Valeur_mesuree;
  U_Accumulateur = (U_Accumulateur * Correction_U_Accumulateur);
  //------------------- Mesure U sur VIN carte Arduino -----------------
  Mesurer_une_tension(Entree_U_NANO); U_Arduino = Valeur_mesuree * Correction_U_Arduino;
  //------------------------ Calcule I sur la 68ohm --------------------
  I_sur_charge = 0; // Suite aux essais on adopte pour la LED verte 10mA et uniquement si T1 et T2.
  if (digitalRead(Pilotage_T2 )) I_sur_charge = U_Solaire_apres_shunt / Resistance_de_charge;
  if (digitalRead(Pilotage_T1 ))
      I_sur_charge = I_sur_charge + 10.0  + (U_Solaire_apres_shunt / Resistance_de_charge);
  //---------------- Calcule I sur sur VIN carte Arduino ---------------
  I_Carte_NANO = 27; if (U_Solaire > 12.9) I_Carte_NANO = 31;
  if (U_Solaire > 15) I_Carte_NANO = 35;
  if (Mode_VEILLE) I_Carte_NANO = I_Carte_NANO - 10;
  //--------------- Calcule I recharge d'un accumulateur ---------------
  // Ici c'est lintensité totale des deux accumulateurs qui est calculée.
  I_Recharge = I_Total_solaire - I_Carte_NANO - I_sur_charge;
  if (I_Recharge < 0) I_Recharge = 0;}
  
void Encadrement() {u8g.drawFrame(0,0,128,64);};

void CADRE_et_HEURE() { 
  CADRES(); Aff_TEXTE_X_Y(7, 11, 866, 3);
  u8g.print(rtc.getDay()); ESPACE(); u8g.print(rtc.getMonth()); Aff_TEXTE(868, 3); u8g.print(rtc.formatTime());}
 
void CADRES() {Encadrement(); u8g.drawFrame(0,13,128,38);} // Cadre TITRE et cadre données.

void Double_cadre() {u8g.drawRFrame(0,0,128,64,10); u8g.drawRFrame(5,5,118,54,5);}

void Encadrement_double() {Double_cadre(); u8g.setScale2x2();}

void PAGE0() {// Affiche la page d'accueil.
  u8g.firstPage(); do {// Début de la boucle d'affichage de la page écran.
  //------------------------------------------------------------------
  Encadrement_double(); Aff_TEXTE_X_Y(6, 13, 875, 9); u8g.undoScale();
  u8g.drawStr(17, 40, Version_PGM); Aff_TEXTE_X_Y(17, 53, 802, 16);}
  //--------------------------------------------------------------
  while(u8g.nextPage());} // Fin de la boucle d'affichage de la page écran.
  
void PAGE1() {// Affiche la page de la tension, l'intensité et la Puissance "solaire". 
  u8g.firstPage(); do {// Début de la boucle d'affichage de la page écran.
  //------------------------------------------------------------------
  CADRE_et_HEURE();
  Aff_TEXTE_X_Y(11, 25, 973, 13); u8g.print(U_Solaire,1); AFF_v();
  Aff_TEXTE_X_Y(11, 36, 960, 13); u8g.print(I_Total_solaire,0); AFF_mA();
  Aff_TEXTE_X_Y(11, 47, 910, 13); u8g.print(U_Solaire * I_Total_solaire / 1000,2); u8g.print('W');
  Aff_TEXTE_X_Y(11, 61, 923, 11); u8g.print(W_Journaliere,0); Aff_TEXTE(871,3);}
  //--------------------------------------------------------------
  while(u8g.nextPage());} // Fin de la boucle d'affichage de la page écran.

void PAGE2() {// Affiche la page des Intensités.
  u8g.firstPage(); do {// Début de la boucle d'affichage de la page écran.
  //------------------------------------------------------------------
  CADRE_et_HEURE(); Trace_resistance(25, 25);
  Aff_TEXTE_X_Y(77, 25, 945, 2);
  // Afficher l'Intensité ou la Tension en fonction de l'inverseur "des piles" :
  if (digitalRead(INV_piles)) {u8g.print(I_sur_charge,0); AFF_mA(); Aff_TEXTE_X_Y(11, 25, 960,1);}
  else {u8g.print(U_Solaire_apres_shunt,1); AFF_v(); Aff_TEXTE_X_Y(11, 25, 1008,1);}
  Aff_TEXTE_X_Y(11, 36, 986, 13);
  u8g.print(I_Carte_NANO,1); AFF_mA(); Aff_TEXTE_X_Y(11, 47, 947, 13);
  u8g.print(I_Recharge,0); AFF_mA();
  /* Problème non résolu : Si la ligne ci-dessus est remplacée par :
     if (I_Recharge == 0) u8g.print("Debit"); else {u8g.print(I_Recharge,0); AFF_mA();}
     l'affichage devient inchohérent quand I_Recharge = 0 ? ? ? */
  Aff_TEXTE_X_Y(11, 61, 897, 13); u8g.print(I_Total_solaire,0); AFF_mA();}
  //--------------------------------------------------------------
  while(u8g.nextPage());} // Fin de la boucle d'affichage de la page écran.

void AFF_MAX_min() {
  if (Octet_EEPROM < 10) ESPACE(); u8g.print(Octet_EEPROM); ESPACE();}
  
void PAGE3() {// Affiche la page des Tensions.
  u8g.firstPage(); do {// Début de la boucle d'affichage de la page écran.
  //------------------------------------------------------------------
  CADRE_et_HEURE(); // Effectue les mesures électrique et calcule les courants.
  Aff_TEXTE_X_Y(11, 25, 884, 13); u8g.print(U_Solaire,1); AFF_v();
  Aff_TEXTE_X_Y(11, 36, 986, 13); u8g.print(U_Arduino,2); AFF_v();
  Mesure_U_Pile_Horloge(); Aff_TEXTE_X_Y(11, 47, 999, 13);
  if (digitalRead(Presence_Strap)) {u8g.print(U_Pile_Horloge,2 ); AFF_v();}
     else  Aff_TEXTE(1012, 4);
  Aff_TEXTE_X_Y(11, 61, 1016, 8); Affiche_caractere_degre();
    if (digitalRead(INV_piles)) AFF_1(); else AFF_2();
    Aff_TEXTE(944,3); u8g.print(U_Accumulateur,1); AFF_v();}
  //--------------------------------------------------------------
  while(u8g.nextPage());} // Fin de la boucle d'affichage de la page écran.
 
float Energie_semaine(){
   float W_Semaine; W_Semaine = 0; PTR = 8;
   for (byte I=1; I<8; I++) {W_Semaine = W_Semaine + Lire_un_Float_en_EEPROM(PTR); PTR = PTR + 4;}
   return W_Semaine / 1000;}
 
int Calcule_Ordre_du_Jour(byte Jour, byte Mois) {
  // Ordre du jours dans l'année : Position temporelle de 1 à 365.
  int Ordre_de_J; Ordre_de_J = 0;
  switch (Mois) {
      case 2 : {Ordre_de_J = 31; break;} // Au premier Février il s'est écoulé 31 jours.
      case 3 : {Ordre_de_J = 59; break;} // Au premier Mars il s'est écoulé 59 jours.
      case 4 : {Ordre_de_J = 90; break;} // Au premier Avril il s'est écoulé 90 jours.
      case 5 : {Ordre_de_J = 120; break;} // Au premier Mai il s'est écoulé 90 jours ...
      case 6 : {Ordre_de_J = 151; break;}
      case 7 : {Ordre_de_J = 181; break;}
      case 8 : {Ordre_de_J = 212; break;}    // NOTE :
      case 9 : {Ordre_de_J = 243; break;}    // Ordre du jour = Nb jours dans les mois précédents
      case 10 : {Ordre_de_J = 273; break;}   // plus Ordre du jour dans le mois courant.
      case 11 : {Ordre_de_J = 304; break;}
      case 12 : Ordre_de_J = 334;}
  Ordre_de_J = Ordre_de_J + Jour;
  return Ordre_de_J;}
  
 void PAGE4() {// Affiche la page des bilans énergétiques.
  int J_dans_annee; J_dans_annee = Calcule_Ordre_du_Jour(rtc.getDay(), rtc.getMonth());
  byte X; X = 5; if (J_dans_annee < 100) X = 8; if (J_dans_annee < 10) X = 12;
  u8g.firstPage(); do {// Début de la boucle d'affichage de la page écran.
  //------------------------------------------------------------------
  CADRES();
  Aff_TEXTE_X_Y(11, 11, 795, 7); u8g.print(Lire_un_Octet_en_EEPROM(2)); ESPACE();
  u8g.print(Lire_un_Octet_en_EEPROM(3)); Aff_TEXTE(677, 4); u8g.print(Lire_un_Entier_en_EEPROM(0));
  Aff_TEXTE_X_Y(11, 25, 818, 10); u8g.print(Lire_un_Float_en_EEPROM(32),0); AFF_mAH();
  Aff_TEXTE_X_Y(11, 36, 828, 10); u8g.print(Energie_semaine(),1); AFF_AH();
  Aff_TEXTE_X_Y(11, 47, 840, 10); u8g.print(Lire_un_Float_en_EEPROM(4) / 1000,1); AFF_AH();
  Aff_TEXTE_X_Y(X, 61, 850, 16); Affiche_caractere_degre();
  u8g.print(Lire_un_Octet_en_EEPROM(44));}
  //--------------------------------------------------------------
  while(u8g.nextPage());} // Fin de la boucle d'affichage de la page écran.
  
void Cadre_semaine_annee() {// Affiche l'invariant des pages Semaine et Année.
  Encadrement(); u8g.setPrintPos(4, 61); u8g.print('T');
  u8g.drawHLine(10,57,8); u8g.drawLine(14,54,16,56); u8g.drawLine(14,60,16,58);}
  
void PAGE5() {// Affiche la page des moyennes et extrémums. (Double emploi de la variable Valeur_mesuree.) 
  float Valeur; byte X;
  u8g.firstPage(); do {// Début de la boucle d'affichage de la page écran.
  //------------------------------------------------------------------
  CADRES(); u8g.drawVLine(64,0,14); u8g.setPrintPos(10, 11);
  u8g.setPrintPos(10, 11);
  switch (rtc.getWeekday()) {
    case 1: {ESPACE(); Aff_TEXTE(691, 5); break;}
    case 2: {ESPACE();Aff_TEXTE(696, 5); break;}
    case 3: {Aff_TEXTE(701, 8); break;}
    case 4: {ESPACE();Aff_TEXTE(709, 5); break;}
    case 5: {Aff_TEXTE(714, 8); break;}
    case 6: {Aff_TEXTE(722, 6); break;}
    case 7: Aff_TEXTE(728, 8);}
  X = 70; if (Num_de_la_semaine < 10) X = 73; Aff_TEXTE_X_Y(X, 11, 672, 5); 
  Affiche_caractere_degre(); ESPACE(); u8g.print(Num_de_la_semaine);
  Affiche_Jour_espace(11, 24); Aff_TEXTE(754, 3); PTR = 40; Affiche_PTR_avec_4_chiffres();
  Affiche_Jour_espace(11, 34); Aff_TEXTE(757, 3); PTR = 36; Affiche_PTR_avec_4_chiffres();
  
  Aff_TEXTE_X_Y(11, 46, 685, 6); Aff_TEXTE(929, 2);
  if (Lire_un_Entier_en_EEPROM(44) == 0) Valeur_mesuree = 0.0; // Ne pas diviser par zéro.
  else Valeur_mesuree =  Lire_un_Float_en_EEPROM(4) / Lire_un_Entier_en_EEPROM(44);
  Affiche_avec_4_chiffres(); Aff_TEXTE_X_Y(23, 61, 664, 8);
  Separe_avec_deux_points(); // Ci dessus laisser la chaîne de caractères car plus rentable
  // que d'utiliser des petites chaînes disponibles en EEPROM.
  u8g.print(Lire_un_Entier_en_EEPROM(44));}
  //-------------------------------------------------------------------
  while(u8g.nextPage());} // Fin de la boucle d'affichage de la page écran.
  
void Affiche_PTR_avec_4_chiffres() {
  Valeur_mesuree = Lire_un_Float_en_EEPROM(PTR); Affiche_avec_4_chiffres();} 
  
void Affiche_avec_4_chiffres() {
  Separe_avec_deux_points();
  if (Valeur_mesuree < 1000) ESPACE(); if (Valeur_mesuree < 100) ESPACE();
  if (Valeur_mesuree < 10) ESPACE(); u8g.print(Valeur_mesuree,0); AFF_mAH();} 
  
void PAGE6() {// Affiche la page pour l'année.
  byte Hauteur,L; L = 23; Hauteur = Calcule_Ordre_du_Jour(rtc.getDay(), rtc.getMonth());
  if (Hauteur < 100) L = 17; if (Hauteur < 10) L = 11;
  u8g.firstPage(); do {// Début de la boucle d'affichage de la page écran.
  //------------------------------------------------------------------
  Cadre_semaine_annee(); u8g.setColorIndex(0); u8g.drawHLine(99,0,L); u8g.setColorIndex(1);
  u8g.setPrintPos(98, 8); AFF_J(); u8g.print(Hauteur);
  u8g.drawVLine(19,3,49); for (byte I=3; I<45; I=I+10) u8g.drawHLine(17,I,2);
  Aff_TEXTE_X_Y(4, 12, 765, 2); Aff_TEXTE_X_Y(4, 22, 767, 2); Aff_TEXTE_X_Y(4, 32, 769, 2);
  Aff_TEXTE_X_Y(4, 42, 771, 2); Aff_TEXTE_X_Y(4, 32, 769, 2); u8g.setPrintPos(10, 52); AFF_zero();
  for (byte I=8; I<50; I=I+10) u8g.drawHLine(16,I,108); u8g.drawVLine(123,8,44);
  for (byte I=21; I<122; I=I+2) u8g.drawVLine(I,49,1); for (byte I=27; I<125; I=I+8) u8g.drawVLine(I,50,2);
  u8g.setColorIndex(0); u8g.drawHLine(20,8,15); u8g.setColorIndex(1);
  // NOTE : Utiliser des lettres disponibles en EEPROM conduit à exactement la même
  //        taille de programme et de données dynamiques. Donc conservé pour lisibilité.
  u8g.setPrintPos(21, 61); AFF_J(); u8g.setPrintPos(30, 61); u8g.print('F');
  u8g.setPrintPos(38, 61); AFF_M(); u8g.setPrintPos(48, 61); AFF_A();
  u8g.setPrintPos(56, 61); AFF_M(); u8g.setPrintPos(64, 61); AFF_J();
  u8g.setPrintPos(73, 61); AFF_J(); u8g.setPrintPos(83, 61); AFF_A();
  u8g.setPrintPos(93, 61); AFF_S(); u8g.setPrintPos(101, 61); u8g.print('O');
  u8g.setPrintPos(109, 61); AFF_N(); u8g.setPrintPos(117, 61); AFF_D();
  PTR = 424; // Tracer le graphe annuel pour les 52 semaines :
  for (byte Semaine = 1; Semaine < 104; Semaine = Semaine + 2) { // 104 = 52 x 2.
      Hauteur = byte (Lire_un_Float_en_EEPROM(PTR)); PTR = PTR + 4;
      u8g.drawLine(Semaine + 19, 48, Semaine + 19, 48 - Hauteur);
      u8g.drawLine(Semaine + 20, 48, Semaine + 20, 48 - Hauteur);}
   u8g.setPrintPos(22, 10); AFF_A(); AFF_H();}
  //-------------------------------------------------------------------
  while(u8g.nextPage());} // Fin de la boucle d'affichage de la page écran.

void ESPACE() {u8g.print(' ');}
void AFF_zero() {u8g.print('0');} // Si un seul caractère ne pas utiliser Aff_TEXTE(ADR, 1) car
void AFF_1() {u8g.print('1');}    // conduit à exactement le même résultat pour une lisibilité
void AFF_2() {u8g.print('2');}    // moindre du listage source.
void AFF_6() {u8g.print('6');}
void AFF_A() {u8g.print('A');}
void AFF_D() {u8g.print('D');}
void AFF_H() {u8g.print('H');}
void AFF_J() {u8g.print('J');}
void AFF_M() {u8g.print('M');}
void AFF_N() {u8g.print('N');}
void AFF_S() {u8g.print('S');}
void AFF_v() {u8g.print('v');}

void AFF_mAH() {ESPACE(); AFF_mA(); AFF_H();}
void AFF_mA() {Aff_TEXTE(871, 2);}
void AFF_AH() {Aff_TEXTE(837, 3);}
void Separe_avec_deux_points() {Aff_TEXTE(810, 3);}
void Affiche_Jour_espace(byte X, byte Y) {Aff_TEXTE_X_Y(X , Y, 860, 5);}
  
void Affiche_caractere_degre() {u8g.print(char(176));}
  
void PAGE7() {// Affiche la page pour la semaine.
  byte Hauteur;
  u8g.firstPage(); do {// Début de la boucle d'affichage de la page écran.
  //------------------------------------------------------------------
  Cadre_semaine_annee();
  u8g.setPrintPos(4, 51); AFF_zero();
  u8g.setPrintPos(4, 39);AFF_2();
  u8g.setPrintPos(4, 27); u8g.print('4');
  u8g.setPrintPos(4, 15); AFF_6();
  for (byte I=12; I<50; I=I+12) u8g.drawHLine(11,I,105);
  for (byte I=6; I<44; I=I+12) u8g.drawHLine(14,I,102);
  for (byte I=17; I<117; I=I+14) u8g.drawVLine(I,3,49);
  //++++++++++++++ Affichage des lettres représentant les jours de la semaine ++++++++++++++
  // NOTE : Utiliser des lettres disponibles en EEPROM conduit à exactement la même
  //        taille de programme et de données dynamiques. Donc conservé pour lisibilité.
  // Dans cette séquence Hauteur sert d'indice dans la boucle "for".
  Hauteur = (rtc.getWeekday())-1; // Hauteur devient un pointeur dans la table des caractères;
  for (byte J=1; J < 8; J++) {
      if (Hauteur > 6) Hauteur = 0;
      switch (J) { 
        case 1: {Affiche_J_sur_le_graphe(22, Hauteur); break;}
        case 2: {Affiche_J_sur_le_graphe(36, Hauteur); break;}
        case 3: {Affiche_J_sur_le_graphe(50, Hauteur); break;}
        case 4: {Affiche_J_sur_le_graphe(64, Hauteur); break;}
        case 5: {Affiche_J_sur_le_graphe(78, Hauteur); break;}
        case 6: {Affiche_J_sur_le_graphe(92, Hauteur); break;}
        case 7: {Affiche_J_sur_le_graphe(106, Hauteur); break;}}
      Hauteur++;}
  //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  u8g.setPrintPos(119, 27); AFF_A(); u8g.setPrintPos(119, 38); AFF_H();
  PTR = 8; // Tracer le graphe pour la semaines :  
  for (byte Jour = 18; Jour < 104; Jour = Jour + 14) {
      Hauteur = byte (round((Lire_un_Float_en_EEPROM(PTR) / 160))); PTR = PTR + 4;
      if (Hauteur > 45) Hauteur = 45; 
      for (byte I = 0; I < 13; I++) u8g.drawLine(Jour + I, 48, Jour + I, 48 - Hauteur);}}
  //------------------------------------------------------------------
  while(u8g.nextPage());} // Fin de la boucle d'affichage de la page écran.
  
void Affiche_J_sur_le_graphe(byte X, byte J) {u8g.setPrintPos(X, 59); u8g.print(JMMJVSD[J]);}

void Cadre_journee_24H() {// Affiche l'invariant des pages Journalières.
  Encadrement(); u8g.setPrintPos(16, 9); AFF_mAH();
  for (byte I=18; I<52; I=I+8) u8g.drawHLine(15,I,107);
  u8g.drawHLine(15,10,92); u8g.drawVLine(19,7,46); u8g.setPrintPos(6, 59); u8g.print('0');
  u8g.drawLine(12,53,14,51); u8g.drawVLine(12,54,7); u8g.drawHLine(5,60,7);
  for (byte I=14; I<48; I=I+8) u8g.drawHLine(17,I,2); for (byte I=25; I<121; I=I+18) u8g.drawVLine(I,48,5);
  for (byte I=25; I<125; I=I+6) u8g.drawVLine(I,51,1); for (byte I=2; I<51; I=I+2) u8g.drawPixel(61,I);
  Aff_TEXTE_X_Y(3, 14, 763, 2); Aff_TEXTE_X_Y(3, 30, 767, 2); Aff_TEXTE_X_Y(3, 45, 771, 2);
  u8g.setPrintPos(23, 62); AFF_6(); u8g.setPrintPos(41, 62); u8g.print('9'); Aff_TEXTE_X_Y(56, 62, 773, 2);
  Aff_TEXTE_X_Y(74, 62, 775, 2); Aff_TEXTE_X_Y(92, 62, 778, 2); Aff_TEXTE_X_Y(110, 62, 774, 2);
  //---------- Ci-dessous : Trace le petit chiffre 5. ----------
  u8g.drawHLine(17,54,4); u8g.drawHLine(17,56,3); u8g.drawVLine(20,57,2); u8g.drawHLine(18,59,2);
  u8g.drawPixel(17,55); u8g.drawPixel(17,58);
  //--------------- Coût : 80 octetsde programme ---------------
  u8g.setColorIndex(0); u8g.drawHLine(27,0,14); u8g.setColorIndex(1);}
  
void Trace_graphe_journee(int PTR) {
  // Trace le graphe correspondant aux 90 échantillons d'une journée.
  byte Valeur;
  for (byte I=0; I<90; I++) {
      Valeur = (Lire_un_Octet_en_EEPROM(PTR) / 5); PTR++;
      u8g.drawLine(I + 31, 50 - (Valeur * 0.8), I + 31, 50);}}
  
void PAGE12() {// Affiche le jour Jr.
  u8g.firstPage(); do {// Début de la boucle d'affichage de la page écran.
  //------------------------------------------------------------------
  Cadre_journee_24H(); Aff_TEXTE_X_Y(112, 11, 929, 2);
  Trace_graphe_journee(64);}
  //------------------------------------------------------------------
  while(u8g.nextPage());} // Fin de la boucle d'affichage de la page écran.
  
void PAGE11() {// Affiche la journée de la veille. (J-1)
  u8g.firstPage(); do {// Début de la boucle d'affichage de la page écran.
  //------------------------------------------------------------------
  Cadre_journee_24H(); Aff_TEXTE_X_Y(109, 11, 760, 3); Trace_graphe_journee(154);}
  //------------------------------------------------------------------
  while(u8g.nextPage());} // Fin de la boucle d'affichage de la page écran.
  
void PAGE8() {// Affiche le graphe du jour spécial.
  u8g.firstPage(); do {// Début de la boucle d'affichage de la page écran.
  //------------------------------------------------------------------
  Cadre_journee_24H(); u8g.setPrintPos(109, 11); u8g.print("Jsp");
  u8g.setPrintPos(21, 22); u8g.print(Lire_un_Octet_en_EEPROM(52));
  u8g.setPrintPos(21, 31); u8g.print(Lire_un_Octet_en_EEPROM(53));
  Trace_graphe_journee(334);}
  //------------------------------------------------------------------
  while(u8g.nextPage());} // Fin de la boucle d'affichage de la page écran.
  
void PAGE9() {// Affiche le jour MAXimal.
  u8g.firstPage(); do {// Début de la boucle d'affichage de la page écran.
  //------------------------------------------------------------------
  Cadre_journee_24H(); Aff_TEXTE_X_Y(109, 11, 754, 3); Trace_graphe_journee(244);}
  //------------------------------------------------------------------
  while(u8g.nextPage());} // Fin de la boucle d'affichage de la page écran.
  
void PAGE10() {// Affiche la page des Extrèmes.
  u8g.firstPage(); do {// Début de la boucle d'affichage de la page écran.
  //------------------------------------------------------------------
  u8g.setRot90(); u8g.drawHLine(0,0,64); u8g.drawHLine(0,15,64);
  u8g.drawXBM(0, 22, Largeur_du_dessin, Hauteur_du_dessin, DessinBitMap);
  Aff_TEXTE_X_Y(4, 12, 654, 10); Aff_TEXTE_X_Y(5, 102, 754, 3); ESPACE();
  Octet_EEPROM = Lire_un_Octet_en_EEPROM(46);
  AFF_MAX_min(); Octet_EEPROM = Lire_un_Octet_en_EEPROM(47); AFF_MAX_min();
  Aff_TEXTE_X_Y(5, 115, 757, 3); ESPACE(); Octet_EEPROM = Lire_un_Octet_en_EEPROM(48);
  AFF_MAX_min(); Octet_EEPROM = Lire_un_Octet_en_EEPROM(49); AFF_MAX_min();
  u8g.setPrintPos(47, 37); u8g.print(Lire_un_Octet_en_EEPROM(50));} // Nb RESETs depuis la RAZ.
  //--------------------------------------------------------------
  while(u8g.nextPage()); // Fin de la boucle d'affichage de la page écran.
  u8g.undoRotation();} // Restitue l'écran horizontal.
  
void Recopie_90_Octets(int Origine, int Destination) {
  for (byte I=0; I<90; I++) {
      Ecrire_un_Octet_en_EEPROM(Destination, Lire_un_Octet_en_EEPROM(Origine)); Origine++; Destination++;}}

byte Calcule_le_Num_de_la_semaine() {      
  Num_de_la_semaine = byte(Calcule_Ordre_du_Jour(rtc.getDay(), rtc.getMonth()) / 7) + 1;
  return Num_de_la_semaine;}
  
void Remise_a_ZERO_des_donnees() {// Force le RESET "chaud" sur le P.E.P.
  Ecrire_un_Entier_en_EEPROM(0, Calcule_Ordre_du_Jour(rtc.getDay(), rtc.getMonth()));
  Ecrire_un_Octet_en_EEPROM(2, rtc.getDay()); Ecrire_un_Octet_en_EEPROM(3, rtc.getMonth());
  PTR = 4; do Ecrire_un_Octet_en_EEPROM(PTR, 0); while (PTR < 632); BIP();
  Ecrire_un_Float_en_EEPROM(36, 7000.0);} // Journée minimale forcée à une valeur maximale pour que
                                          // la mise à jour puisse se faire lors du changement de journée.

void Surveiller_des_Variables() {
//  Reveiller_Ecran(); u8g.firstPage(); do{
//  u8g.setPrintPos(4, 15); u8g.print("variable 1 = "); u8g.print(xxx);
//  u8g.setPrintPos(4, 30); u8g.print("variable 2 = "); u8g.print(xxx);
//  u8g.setPrintPos(4, 45); u8g.print("variable 3 = "); u8g.print(xxx);
//  u8g.setPrintPos(4, 60); u8g.print("variable 4 = "); u8g.print(xxx);}
}//  while(u8g.nextPage()); ATTENDRE_un_BP();}

