/* Simule le compteur de millis() pour vérifier la durée avant recyclage et le comportement
   lorsque le compteur binaire repasse à zéro.
   
   À tout moment cliquer sur l'un de des B.P. du clavier fige le déroulement du programme.
   Passer les deux lignes @@@@@@@@@@@@@@@@@@@@ en remarque permet de tester le blocage
   du programme lors du recyclage du compteur binaire.

 Programme : 10332 octets 33%  Mémoire dynamique :  497 octets 24%    */

//--------------------- Déclarations pour le matériel -----------------------
#include "U8glib.h"
U8GLIB_SH1106_128X64 u8g(U8G_I2C_OPT_NONE); // Usage de la ligne I2C (Two Wire Interface)

#include <avr/sleep.h>
#include <Wire.h>

//--------------------- Définition des broches d'E/S ------------------------
#define BUZZER 2 // Pilotage du bruiteur.
// NOTE : pas besoin de déclarer D3 car traitée par la bibliothèque de l'Horloge/Calendrier.
#define Sommeil_interdit 4 // Entrée binaire D4 pour Sommeil autorisé.
#define Pilotage_T1 5
#define Pilotage_T2 6
#define Presence_Strap 7 // Entrée binaire D7 pour tester la présence d'un "Strap".
#define RESET_froid 8 // Entrée binaire D8 pour choisir entre RESET Froid ou P.E.P.
#define INV_piles 9  // Inverseur de mesures de la tension sur les piles..
#define LED_Clavier 10 // Témoin d'activation d'un B.P.
#define LED_Rouge 11  // LED B.P. long.
#define LED_bleue 12 // Témoin de la boucle de base.
#define LED_Arduino 13 // Utilisée parfois en tests de mise au point du programme.

#define Entree_U_Solaire 1 // Entrée analogique A1 pour mesurer la tension cellules solaires.
#define Entree_U_apres_10ohm 2 // Entrée analogique A2 pour mesurer la tension après la 10ohm.
#define Entree_U_Pile_Horloge 3 // Entrée analogique A3 pour mesurer la tension pile de sauvegarde.
#define Entree_U_Accumulateur 6    // Entrée analogique A6 pour mesurer la tension sur l'accumulateur.
#define Entree_U_NANO 7    // Entrée analogique A7 pour mesurer la tension sur la carte NANO.

//-------------------- Déclaration des constantes --------------------
#define NB_tests_antirebonds 50 // Nombre de tests de stabilisation B.P.
//-------------------- Déclaration des variables ---------------------
boolean Clic_sur_le_BP, Augmenter_le_choix, Clic_long;

int NB_jours;
byte Compteur_Binaire;
byte Diviseur_par_3;

byte Octet_EEPROM;

int PTR; // Pointeur pour échanger des Octets en EEPROM.

int Ancienne_valeur_Temps_Ecoule_depuis_RESET;

void setup() {
  u8g.setFont(u8g_font_profont12); // Police de matrices de 6x10 pixels lettres MAJ/min comprises.
  // ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  pinMode(LED_bleue, OUTPUT); pinMode(BUZZER, OUTPUT); digitalWrite(BUZZER, LOW); pinMode(LED_Rouge, OUTPUT);
  digitalWrite(LED_Rouge, LOW); pinMode(LED_Clavier, OUTPUT);  digitalWrite(LED_Clavier, LOW);
  pinMode(Pilotage_T1, OUTPUT);  digitalWrite(Pilotage_T1, LOW); pinMode(Pilotage_T2, OUTPUT);
  digitalWrite(Pilotage_T2, LOW); pinMode(Presence_Strap, INPUT);
  digitalWrite(Presence_Strap, HIGH); // Valide la résistance de pullup.
  pinMode(RESET_froid, INPUT); digitalWrite(RESET_froid, HIGH); // Valide la résistance de pullup.
  pinMode(Sommeil_interdit, INPUT); digitalWrite(Sommeil_interdit, HIGH); // Valide la résistance de pullup.
  pinMode(INV_piles, INPUT); digitalWrite(INV_piles, HIGH); // Valide la résistance de pullup.
  // ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  NB_jours = 0; Compteur_Binaire = 1; Diviseur_par_3 = 0; Ancienne_valeur_Temps_Ecoule_depuis_RESET = 0;}

void loop() { 
  if (Compteur_Binaire > Ancienne_valeur_Temps_Ecoule_depuis_RESET)  {
    // ++++++++++++++++++++++++++ Réalisé rapidement. +++++++++++++++++++++++++
    digitalWrite(LED_bleue, HIGH); delay(3); digitalWrite(LED_bleue, LOW);
    NB_jours++; Diviseur_par_3++;
    if(Diviseur_par_3 != 3) {Compteur_Binaire++; Ancienne_valeur_Temps_Ecoule_depuis_RESET++;}
    else Diviseur_par_3 = 0;
    Surveiller_des_Variables();
    // --------------- Traiter la saturation du pseudo compteur  millis(). -----------------
    if ((Ancienne_valeur_Temps_Ecoule_depuis_RESET - Compteur_Binaire) > 100) // @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    {Ancienne_valeur_Temps_Ecoule_depuis_RESET = 0; Compteur_Binaire = 1;}}   // @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
  
  Tester_le_BP_A0();
  if (Clic_sur_le_BP) {Clic_sur_le_BP = false;}} // Action sur le B.P. prise en compte.
  //============================== Fin de la boucle de base ===========================
     
//--------------------- Routines pour la gestion EEPROM ---------------------
void Aff_TEXTE_X_Y(byte X, byte Y, int PTR, byte Longueur) {
  u8g.setPrintPos(X, Y); Aff_TEXTE(PTR, Longueur);}

void Aff_TEXTE(int PTR, byte Longueur) {
  for (byte I=0; I<Longueur; I++) {u8g.print(char(Lire_un_Octet_en_EEPROM(PTR))); PTR++;}; }
  
byte Lire_un_Octet_en_EEPROM(int PTR) {
  return eeprom_read_byte((unsigned char*) PTR);}

void BIP() {tone(BUZZER,3000,100); digitalWrite(BUZZER, LOW);}
  
void ATTENDRE_un_BP() { Clic_sur_le_BP = false;
  while (!Clic_sur_le_BP) {
    digitalWrite(LED_Clavier,!digitalRead(LED_Clavier)); // Inverser la LED clavier.
    delay(50); Tester_le_BP_A0();}
  digitalWrite(LED_Clavier,LOW); Clic_sur_le_BP = false; }
  
// >>>>>>>>>>>>>>> Le B.P. est câblé en "Analogique" <<<<<<<<<<<<<<<<<<
void Tester_le_BP_A0() { // La lecture sur A0 donne 0 ou 930 maximum.
  int Numerisation_BP; // Valeur retournée par la CAN de A0.
  byte COMPTEUR; unsigned long Horloge_actuelle;
  Numerisation_BP = analogRead(0);
  if ((Numerisation_BP > 710) || (Numerisation_BP < 240)) {
    Clic_sur_le_BP = true; digitalWrite(LED_Clavier, HIGH);
    Augmenter_le_choix = false;
    Horloge_actuelle = millis(); Clic_long = false;
    if (Numerisation_BP > 710) { Augmenter_le_choix = true; }
    // Attendre_la_stabilisation du travail durant 100mS. Bruiteur à 500Hz et laisse D2 à zéro :
    for(byte I=0; I < 50; I++) {digitalWrite(BUZZER, !digitalRead(BUZZER)); delay(1);}  
    // Attendre_la_stabilisation du repos :
    COMPTEUR = 0;
    while (COMPTEUR < NB_tests_antirebonds) {
      if ((millis() - Horloge_actuelle) > 700) {Clic_long = true; digitalWrite(LED_Rouge, HIGH);}
      COMPTEUR++; Numerisation_BP = analogRead(0);
      if ((Numerisation_BP > 710) || (Numerisation_BP < 240)) COMPTEUR = 0; }
  digitalWrite(LED_Clavier, LOW); digitalWrite(LED_Rouge, LOW);}}
             
void Surveiller_des_Variables() {
  u8g.firstPage(); do{
  u8g.setPrintPos(4, 15); u8g.print("Cmpteur = "); u8g.print(Compteur_Binaire);
  u8g.setPrintPos(4, 30); u8g.print("Nb jours = "); u8g.print(NB_jours);
//  u8g.setPrintPos(4, 45); u8g.print("variable 3 = "); u8g.print(xxx);
  u8g.setPrintPos(4, 60); u8g.print("Depuis RESET : "); u8g.print(Ancienne_valeur_Temps_Ecoule_depuis_RESET);}
 while(u8g.nextPage());}

